/*
 * main.c
 *
 *  Created on: 31 janv. 2013
 *      Author: rj092
 *
 * Description of the sample:
 * This sample will start an autorefresh acquisition on the system, and will read the value measured by the system.
 * The autorefresh acquisition will acquire one value each 200ms.
 * You can modify the behavior by changing the parameter of the soap_call_MSXExxxx__AcquisitionAutoRefreshInitAndStart function.
 *
 * It takes in parameter two values:
 * - IP_ADDRESS: the network address of your system
 * - COUNT: The number of packets that you want to acquire (one packet contains the value of all of the channels)
 *
 * To get the number of acquisition channels, the sample uses the function soap_call_MSXExxxx__AnalogOutputGetNumberOfChannels.
 *
 * To stop a sequence acquisition, the sample uses the function soap_call_MSXExxxx__AcquisitionSequenceStopAndRelease
 *
 * To stop an autorefresh acquisition, the sample uses the function soap_call_MSXExxxx__AcquisitionAutoRefreshStopAndRelease
 *
 * To start the autorefresh acquisition, the sample uses the function soap_call_MSXExxxx__AcquisitionAutoRefreshInitAndStart
 */

#include <stdlib.h>
#include <stdio.h>

#include <MSXExxxx.nsmap>

/* defines the type of the acquisition units */
const char * typestr [] = { "Not available","Temperature channel","Pressure channel","Transducer channel","Analog input channel","Analog input ICP channel","Digital I/O port" };

static void usage()
{
	printf("./autorefresh IP_ADDRESS COUNT \n");
	printf("- IP_ADDRESS: IP address of the system.\n");
	printf("- COUNT: Number of packet to acquire. (1 to 1000)\n");
	exit(1);
}

static void soap_cleanup_memory(struct soap * env)
{
	soap_destroy(env);
	soap_end(env);
	soap_free(env);
}

/**
 * Stop any running acquisition
 * It does not check the return value of the function (only the connection state)
 */
static void stop_acquisition(struct soap * env, char * soapip )
{
	struct MSXExxxx__Response response;
	int retval = 0;

	retval = soap_call_MSXExxxx__AcquisitionAutoRefreshStopAndRelease(env, soapip, NULL, NULL, &response);
	if ( retval != 0 )
	{
		printf("SOAP connection error, or function not available on the system (%d).\n", retval);
		soap_cleanup_memory(env);
		exit(-1);
	}

	retval = soap_call_MSXExxxx__AcquisitionSequenceStopAndRelease(env, soapip, NULL, NULL, &response);
	if ( retval != 0 )
	{
		printf("SOAP connection error, or function not available on the system (%d).\n", retval);
		soap_cleanup_memory(env);
		exit(-1);
	}
}

/**
 * Get the number of acquisition channels
 */
static unsigned long get_channel_count (struct soap * env, char * soapip )
{
	struct MSXExxxx__unsignedLongResponse response;
	int retval = 0;
	retval = soap_call_MSXExxxx__AcquisitionGetNumberOfChannels( env, soapip, NULL, 0, &response);
	if ( retval != 0 )
	{
		printf("SOAP connection error, or function not available on the system (%d).\n", retval);
		soap_cleanup_memory(env);
		exit(-1);
	}
	if ( response.sResponse.iReturnValue != 0 )
	{
		printf("Error while getting the number of acquisition channels. (iReturnValue: %d, syserrno; %d)", response.sResponse.iReturnValue, response.sResponse.syserrno);
		soap_cleanup_memory(env);
		exit(-1);
	}
	return response.ulValue;
}

/**
 * Initialises the autorefresh acquisition using all the possible channels.
 * This function will also display information concerning the acquired channels
 */
static void start_autorefresh(struct soap * env, char * soapip )
{
	struct MSXExxxx__Response defaultresponse;
	int retval = 0;
	unsigned long i = 0;
	unsigned long channelcount = 0;
	unsigned long channelmask = 0;

	/* get the number of acquisition channels */
	channelcount = get_channel_count(env, soapip);

	/* display informations on these channels and also construct the mask for the autorefresh acquisition */
	for ( i = 0; i < channelcount; i++ )
	{
		struct MSXExxxx__AcquisitionGetChannelInfoResponse channelresponse;
		retval = soap_call_MSXExxxx__AcquisitionGetChannelInfo(env, soapip, NULL, i, 0, &channelresponse);
		if ( retval != 0 )
		{
			printf("SOAP connection error, or function not available on the system (%d).\n", retval);
			soap_cleanup_memory(env);
			exit(-1);
		}
		if ( channelresponse.sResponse.iReturnValue != 0 )
		{
			printf("Error while getting information on an acquisition channel. (iReturnValue: %d, syserrno; %d)\n", channelresponse.sResponse.iReturnValue, channelresponse.sResponse.syserrno);
			soap_cleanup_memory(env);
			exit(-1);
		}

		/* add to the mask */
		channelmask += (1<<i);

		/* display */
		printf("Index : %lu \n", i);
		printf("Hardware position : %lu \n", channelresponse.ulHwPosition);
		printf("Type : %s (%lu) \n\n", typestr[channelresponse.ulType], channelresponse.ulType);
	}

	/* stop any running acquisition */
	stop_acquisition(env, soapip);

	/* start the autorefresh acquisition */
	retval = soap_call_MSXExxxx__AcquisitionAutoRefreshInitAndStart(env, soapip, NULL,
			channelmask,
			1,				/* no average */
			200, 1, 			/* 200 ms of refresh time*/
			0, 0,0,0,0,		/* no trigger */
			17,				/* analog value with timestamp */
			1,				/* force start */
			0, 0, 0, &defaultresponse);
	if ( retval != 0 )
	{
		printf("SOAP connection error, or function not available on the system (%d).\n", retval);
		soap_cleanup_memory(env);
		exit(-1);
	}
	if ( defaultresponse.iReturnValue != 0 )
	{
		printf("Error while starting autorefresh acquisition. (iReturnValue: %d, syserrno; %d)\n", defaultresponse.iReturnValue, defaultresponse.syserrno);
		soap_cleanup_memory(env);
		exit(-1);
	}
}

/**
 * Get the values acquired within the autorefresh acquisition
 */
static void get_and_display_values(struct soap * env, char * soapip, unsigned long count )
{
	unsigned long i = 0;
	unsigned long channelcount = 0;

	/* get the number of acquisition channels */
	channelcount = get_channel_count(env, soapip);

	/* get asked values */
	for ( i = 0; i < count; i++ )
	{
		int retval = 0;
		struct MSXExxxx__AcquisitionAutoRefreshGetValuesResponse response;
		unsigned long j = 0;

		/* get the values */
		retval = soap_call_MSXExxxx__AcquisitionAutoRefreshGetValues(env, soapip, NULL, 1, &response);
		if ( retval != 0 )
		{
			printf("SOAP connection error, or function not available on the system (%d).\n", retval);
			soap_cleanup_memory(env);
			exit(-1);
		}
		if ( response.sResponse.iReturnValue != 0 )
		{
			printf("Error while getting acquired values. (iReturnValue: %d, syserrno; %d)\n", response.sResponse.iReturnValue, response.sResponse.syserrno);
			soap_cleanup_memory(env);
			exit(-1);
		}

		/* display the title line */
		printf("Timestamp(s) \t Timestamp (us) \t");
		for ( j= 0; j < channelcount; j++ )
			printf("Ch %lu \t", j);

		printf("\n");

		/* display timestamp */
		printf("%lu \t\t %07lu \t\t", response.ulTimeStampHigh, response.ulTimeStampLow);

		/* display value of the channels */
		for (j = 0; j < channelcount; j++ )
			printf("%.3f \t", response.dValue[j]);

		printf("\n");
	}
}

/* entry point */
int main (int argc, char * argv[] )
{
	struct soap * env;
	char soapip[255] = {0};
	unsigned long count = 0;

	/* check user arguments*/
	if ( argc != 3 )
		usage();

	/* construct the soap ip */
	snprintf(soapip, 255, "%s:5555",argv[1]);

	count = (unsigned long)atoi(argv[2]);
	if ( (count < 1) || (count > 1000))
		usage();

	/* create SOAP environment */
	if ( (env = soap_new()) == NULL)
	{
		printf("Error while creating SOAP environment.\n");
		exit(1);
	}
	soap_init2(env, SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
	env->send_timeout = 5;
	env->recv_timeout = 5;

	/* start the autorefresh acquisition */
	start_autorefresh(env, soapip);

	/* get and display the values */
	get_and_display_values(env, soapip, count);

	/* stop any acquisition */
	stop_acquisition(env, soapip);

	/* release some resources */
	soap_cleanup_memory(env);

	return 0;
}
