/*
 * main.c
 *
 *  Created on: 29 janv. 2013
 *      Author: rj092
 *
 * Description of the sample:
 * This sample will write a value on all available analog outputs, using a trigger mask (software trigger).
 * Then he waits during a given time (parameter DELAY) and generates a software trigger that will apply the voltage
 * on the outputs.
 *
 * It takes in parameter two values:
 * - IP_ADDRESS: the network address of your system
 * - VALUE: the value that will be written on the analog outputs. This value is an integer between 0 and 65535.
 * 0 represents -10V and 65535 represents +10V
 *
 * To get the number of analog outputs, the sample uses the function soap_call_MSXExxxx__AnalogOutputGetNumberOfChannels.
 *
 * To set the value of an analog output, the sample uses the function soap_call_MSXExxxx__AnalogOutputWrite1Value.
 *
 * To trigger the analog outputs (apply the last value that was written on the output), the samples uses the function soap_call_MSXExxxx__AnalogOutputTriggerOutputs
 *
 * To get the value of an analog output, the sample uses the function soap_call_MSXExxxx__AnalogOutputGetChannelValue
 *
 */
#include <stdlib.h>
#include <stdio.h>

#include <MSXExxxx.nsmap>

static void usage()
{
	printf("./getanaoutvalue IP_ADDRESS VALUE \n");
	printf("- IP_ADDRESS: IP address of the system.\n");
	printf("- VALUE: raw value that will be written on all the analog outputs of the system. (0-65535) \n");
	exit(1);
}

static void soap_cleanup_memory(struct soap * env)
{
	soap_destroy(env);
	soap_end(env);
	soap_free(env);
}

/* entry point */
int main (int argc, char * argv[] )
{
	struct soap * env;
	unsigned long value = 0;
	char soapip[255] = {0};
	struct MSXExxxx__unsignedLongResponse response;
	struct MSXExxxx__Response defaultresponse;
	int retval = 0;
	unsigned long i = 0;
	unsigned long anaoutcount = 0;

	/* check user arguments*/
	if ( argc != 3 )
		usage();

	/* construct the soap ip */
	snprintf(soapip, 255, "%s:5555",argv[1]);

	/* get the value and check it */
	value = (unsigned long) atoi(argv[2]);
	if ( (value < 0) || (value > 0xFFFF) )
	{
		printf("VALUE must be higher than 0 and lower than 0xFFFF\n.");
		exit(1);
	}

	/* Create SOAP environment */
	if ( (env = soap_new()) == NULL)
	{
		printf("Error while creating SOAP environment.\n");
		exit(1);
	}
	soap_init2(env, SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
	env->send_timeout = 5;
	env->recv_timeout = 5;

	/* get the number of available analog outputs */
	retval = soap_call_MSXExxxx__AnalogOutputGetNumberOfChannels( env, soapip, NULL, 0, &response);
	if ( retval != 0 )
	{
		printf("SOAP connection error, or function not available on the system (%d).\n", retval);
		soap_cleanup_memory(env);
		exit(-1);
	}
	if ( response.sResponse.iReturnValue != 0 )
	{
		printf("Error while getting the number of analog output channels. (iReturnValue: %d, syserrno; %d)", response.sResponse.iReturnValue, response.sResponse.syserrno);
		soap_cleanup_memory(env);
		exit(-1);
	}

	/* copy */
	anaoutcount = response.ulValue;

	/* we read the current value of the analog outputs - initialisation state */
	for ( i = 0; i < anaoutcount; i++ )
	{
		double analogvalue = 0.0;
		retval = soap_call_MSXExxxx__AnalogOutputGetChannelValue(env, soapip, NULL, i, 0, &response);
		if ( retval != 0 )
		{
			printf("SOAP connection error, or function not available on the system (%d).\n", retval);
			soap_cleanup_memory(env);
			exit(-1);
		}
		if ( response.sResponse.iReturnValue != 0 )
		{
			printf("Error while getting the number of analog output channels. (iReturnValue: %d, syserrno; %d)", response.sResponse.iReturnValue, response.sResponse.syserrno);
			soap_cleanup_memory(env);
			exit(-1);
		}
		analogvalue = (((20.0 * (double)response.ulValue)) / ((double)0xFFFF)) - 10.0;
		printf("Current value of output %d : 0x%x (%f V)\n", (int)i, (unsigned int)response.ulValue, analogvalue);
	}

	/*
	 * set all analog outputs to the selected value
	 * we use the software trigger (value 4 -> bit 2 set to 1)
	 * polarity is bipolar, that means that 0 will output -10V and 0xFFFF will output 10V
	 * */
	for ( i = 0; i < anaoutcount; i++ )
	{
		double analogvalue = (((20.0 * (double)value)) / ((double)0xFFFF)) - 10.0;

		printf("Set analog output %d to 0x%x (%f V)\n", (int)i, (unsigned int)value, analogvalue);

		/* set the analog output */
		retval = soap_call_MSXExxxx__AnalogOutputWrite1Value(env, soapip, NULL,
				i, 	/* channel */
				0, 					/* voltage*/
				1,					/* bipolar */
				4, 0, 0,			/* trigger mask : software trigger, trigger edge selection and trigger edge count */
				value,				/* value to set */
				0, 0, 0, &defaultresponse);
		if ( retval != 0 )
		{
			printf("SOAP connection error, or function not available on the system (%d).\n", retval);
			soap_cleanup_memory(env);
			exit(-1);
		}
		if ( defaultresponse.iReturnValue != 0 )
		{
			printf("Error while setting analog output value (iReturnValue: %d, syserrno; %d)", defaultresponse.iReturnValue, defaultresponse.syserrno);
			soap_cleanup_memory(env);
			exit(-1);
		}
	}

	/*
	 * we read the current value of the analog outputs
	 * the value did not change since we use the software trigger
	 * */
	for ( i = 0; i < anaoutcount; i++ )
	{
		double analogvalue = 0.0;
		retval = soap_call_MSXExxxx__AnalogOutputGetChannelValue(env, soapip, NULL, i, 0, &response);
		if ( retval != 0 )
		{
			printf("SOAP connection error, or function not available on the system (%d).\n", retval);
			soap_cleanup_memory(env);
			exit(-1);
		}
		if ( response.sResponse.iReturnValue != 0 )
		{
			printf("Error while getting the number of analog output channels. (iReturnValue: %d, syserrno; %d)", response.sResponse.iReturnValue, response.sResponse.syserrno);
			soap_cleanup_memory(env);
			exit(-1);
		}
		analogvalue = (((20.0 * (double)response.ulValue)) / ((double)0xFFFF)) - 10.0;
		printf("Current value of output %d : 0x%x (%f V)\n", (int)i, (unsigned int)response.ulValue, analogvalue);
	}

	/* generate the software trigger to set the last written value on the outputs */
	printf("Generate trigger.\n");
	retval = soap_call_MSXExxxx__AnalogOutputTriggerOutputs(env, soapip, NULL, 0, &defaultresponse);
	if ( retval != 0 )
	{
		printf("SOAP connection error, or function not available on the system (%d).\n", retval);
		soap_cleanup_memory(env);
		exit(-1);
	}
	if ( defaultresponse.iReturnValue != 0 )
	{
		printf("Error while sending software trigger. (iReturnValue: %d, syserrno; %d)", defaultresponse.iReturnValue, defaultresponse.syserrno);
		soap_cleanup_memory(env);
		exit(-1);
	}

	/*
	 * we read the current value of the analog outputs
	 * the value is now changed since we triggered the outputs
	 * */
	for ( i = 0; i < anaoutcount; i++ )
	{
		double analogvalue = 0.0;
		retval = soap_call_MSXExxxx__AnalogOutputGetChannelValue(env, soapip, NULL, i, 0, &response);
		if ( retval != 0 )
		{
			printf("SOAP connection error, or function not available on the system (%d).\n", retval);
			soap_cleanup_memory(env);
			exit(-1);
		}
		if ( response.sResponse.iReturnValue != 0 )
		{
			printf("Error while getting the number of analog output channels. (iReturnValue: %d, syserrno; %d)", response.sResponse.iReturnValue, response.sResponse.syserrno);
			soap_cleanup_memory(env);
			exit(-1);
		}
		analogvalue = (((20.0 * (double)response.ulValue)) / ((double)0xFFFF)) - 10.0;
		printf("Current value of output %d : 0x%x (%f V)\n", (int)i, (unsigned int)response.ulValue, analogvalue);
	}

	/* release some resources */
	soap_cleanup_memory(env);

	return 0;
}
